package com.takensoft.cms.accesCtrl.service.Impl;

import com.takensoft.cms.accesCtrl.dao.AccesCtrlDAO;
import com.takensoft.cms.accesCtrl.service.AccesCtrlService;
import com.takensoft.cms.accesCtrl.vo.AccesCtrlVO;
import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.codeManage.vo.CodeManageVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author  : takensoft
 * @since   : 2024.04.01
 *
 * 접근 제어 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * AccesCtrlService : 접근 제어 관련 인터페이스 상속
 */
@Service("accesCtrlService")
@RequiredArgsConstructor
public class AccesCtrlServiceImpl extends EgovAbstractServiceImpl implements AccesCtrlService {

    private static final Logger LOGGER = LoggerFactory.getLogger(AccesCtrlServiceImpl.class);
    private final JWTUtil jwtUtil;
    private final AccesCtrlDAO accesCtrlDAO;
    private final CodeManageService codeManageService;


    /**
     * @author takensoft
     * @since 2024.04.09
     * 접근 제어 등록
     */
    @Override
    @CacheEvict(value = "routerAccesCheck", allEntries = true)
    public Map<String, Object> accesCtrlSave(AccesCtrlVO accesCtrlVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        accesCtrlVO.setRgtr(jwtUtil.getWriter());
        int saveResult = accesCtrlDAO.save(accesCtrlVO);
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("result", saveResult);
        result.put("acsCntrlId", accesCtrlVO.getAcsCntrlId());
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * 접근 제어 수정
     */
    @Override
    @CacheEvict(value = "routerAccesCheck", allEntries = true)
    public int accesCtrlUpdate(AccesCtrlVO accesCtrlVO) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        accesCtrlVO.setMdfr(jwtUtil.getWriter());
        return accesCtrlDAO.update(accesCtrlVO);
    }

    /**
     * @author takensoft
     * @since 2024.04.12
     * 접근 제어 조회(관리자 페이지용)
     */
    @Override
    public Map<String, Object> findAll(Map<String, String> params) throws Exception {
        Map<String, Object> result = new HashMap<String, Object>(); // 반환 객체 생성
        Pagination search = new Pagination(0, params); // 검색조건 객체 생성
        int cnt = accesCtrlDAO.findAllCnt(search); // 게시물 갯수
        Pagination pagination = new Pagination(cnt, params); // 게시물 갯수 조회 후 검색조건 객쳄 초기화
        List<AccesCtrlVO> list = accesCtrlDAO.findAll(pagination); // 목록 조회
        List<CodeManageVO> codeList = codeManageService.findByChildCdCache("acsCntrlGroup"); // 검색 조건

        result.put("list", list);
        result.put("pagination", pagination);
        result.put("codeList", codeList);
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 접근 제어 조회(관시스템용 - 시큐리티 접근 제어)
     */
    @Override
    @Cacheable(value = "routerAccesCheck", key = "#ip")
    public List<AccesCtrlVO> findAllAccesCtrlSecurity(String ip) throws Exception {
        return accesCtrlDAO.findAllAccesCtrlSecurity(ip);
    }

    /**
     * @author takensoft
     * @since 2024.04.09
     * 접근 제어 상세 조회
     */
    @Override
    public Map<String, Object> findByAccesCtrl(int acsCntrlId) throws Exception {
        Map<String, Object> result = new HashMap<String, Object>(); // 반환 객체 생성
        AccesCtrlVO accesCtrl = new AccesCtrlVO();
        if(acsCntrlId != 0) {
            accesCtrl = accesCtrlDAO.findByAccesCtrl(acsCntrlId);
        }
        List<CodeManageVO> codeList = codeManageService.findByChildCdCache("accesGroup"); // 수정 및 등록에 필요한 접근제어 종류 조회

        result.put("accesCtrl", accesCtrl);
        result.put("codeList", codeList);

        return result;
    }


}
