package com.takensoft.cms.accesCtrl.service.Impl;

import com.takensoft.cms.accesCtrl.service.AccesCtrlService;
import com.takensoft.cms.accesCtrl.service.SystemAccesCtrlService;
import com.takensoft.cms.accesCtrl.vo.AccesCtrlVO;
import com.takensoft.common.util.CommonUtils;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.util.AntPathMatcher;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * @author  : takensoft
 * @since   : 2024.05.20
 *
 * 시스템 접근 제어 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * AuthorService : 시스템 접근 제어 관련 인터페이스 상속
 */
@Service("systemAccesCtrlService")
@RequiredArgsConstructor
public class SystemAccesCtrlServiceImpl extends EgovAbstractServiceImpl implements SystemAccesCtrlService {

    private final CommonUtils commonUtils;
    private final AccesCtrlService accesCtrlService;

    /**
     * @author takensoft
     * @since 2024.04.09
     * 접근 제어 조회(시스템용 - 라우터)
     */
    @Override
    public boolean routerAccesCheck(Map params, HttpServletRequest req) throws Exception {
        String ipAdrs = commonUtils.getIp(req); // 아이피 정보
        String path = params.get("path").toString(); // 접근 경로
        // 공통 접근 경로인 경우
        if(isPath(path, "/cmmn/**")) {
            return true;
        }

        List<AccesCtrlVO> list = accesCtrlService.findAllAccesCtrlSecurity(ipAdrs); // 접근 제어 정보 조회

        // 관리자 경로 접근 검증
        if(isPath(path, "/adm/**")) {
            // 관리자 페이지로 접속하였으나 접근 제어 정보가 없을 경우
            if(list.isEmpty()) {
                return false;
            }
        }
        return matches(list, path);
    }

    /**
     * @author takensoft
     * @since 2024.05.29
     * @param accesCtrlList : 해당 아이피의 접근 제어 정보
     * @param path : 현재 접근 경로
     * 접근 여부 확인
     */
    private boolean matches(List<AccesCtrlVO> accesCtrlList, String path) throws Exception {
        AntPathMatcher matcher = new AntPathMatcher();
        for(AccesCtrlVO vo : accesCtrlList) {
            if(matcher.match(vo.getCntrlCrs(), path)) {
                return "permit".equals(vo.getCntrlType()); // 접근 허용일 경우 true, 그 외 false
            }
        }
        if(matcher.match("/adm/**", path)) {
            return false;
        }
        return true;
    }

    /**
     * @author takensoft
     * @since 2024.05.29
     * @param path : 현재 접근 경로
     * @param url : 확인 경로
     * 경로 확인
     */
    private boolean isPath(String path, String url) {
        AntPathMatcher matcher = new AntPathMatcher();
        return matcher.match(url, path);
    }
}