package com.takensoft.cms.bbs.dao;

import com.takensoft.cms.bbs.vo.BbsCnVO;
import com.takensoft.cms.popup.vo.PopupVO;
import com.takensoft.common.Pagination;
import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import java.util.HashMap;
import java.util.List;

/**
 * @author  : 하석형
 * @since   : 2024.05.09
 *
 * 게시판 내용 관련 Mapper
 */
@Mapper("bbsCnDAO")
public interface BbsCnDAO {
    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 아이디 중복 검사
     */
    public boolean bbsCnIdCheck(BbsCnVO bbsCnVO) throws Exception;

    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 내용 등록
     */
    public int save(BbsCnVO bbsCn) throws Exception;

    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 내용 목록 조회 개수
     */
    public int findAllCnt(Pagination pagination) throws Exception;

    /**
     * @author  하석형
     * @since   2024.05.09
     * 게시판 내용 목록 조회
     */
    public List<BbsCnVO> findAll(Pagination pagination) throws Exception;

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 내용 상세 조회
     */
    public BbsCnVO findByBbsId(String bbsCnId);

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 내용 수정
     */
    public int update(HashMap<String, Object> params);

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 내용 삭제
     */
    public int delete(BbsCnVO bbsCnVO);

    /**
     * @author 방선주
     * @since  2024.05.22
     *
     * 조회수 증가 처리
     */
    public int updateVwCnt(String bbsId) throws Exception;

    /**
     * @author 방선주
     * @since  2024.05.23
     *
     * 이전글
     */
    public BbsCnVO findPrevBbsCn(BbsCnVO bbsCnVO);

    /**
     * @author 방선주
     * @since  2024.05.23
     *
     * 다음글
     */
    public BbsCnVO findNextBbsCn(BbsCnVO bbsCnVO);

    /**
     * @author  : 박정하
     * @since   : 2024.05.28
     *
     * 게시글 최신 목록 조회 (최신글 5건 출력)
     */
    public List<HashMap> selectBbsCnListByNew() throws Exception;

    /**
     * @author  : 방선주
     * @since   : 2024.06.04
     *
     * 질의형 게시판 답변 등록
     */
    public int saveAnswer(BbsCnVO bbsCnVO) throws Exception;

    /**
     * @author  : 방선주
     * @since   : 2024.06.13
     *
     * 최근 공지 5개 조회
     */
    public List<BbsCnVO> findFiveNotice(BbsCnVO bbsCn);
}