package com.takensoft.cms.bbs.dao;

import com.takensoft.cms.bbs.vo.BbsMngVO;
import com.takensoft.cms.bbs.vo.BbsRouterVO;
import com.takensoft.common.Pagination;
import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.05.08
 *
 * 게시판 관리 관련 Mapper
 */
@Mapper("bbsMngDAO")
public interface BbsMngDAO {
    /**
     * @author  박정하
     * @since   2024.05.08
     * 게시판 아이디 중복 검사
     */
    public boolean bbsMngIdCheck(BbsMngVO bbsMngVO) throws Exception;

    /**
     * @author  하석형
     * @since   2024.05.10
     * 게시판 관리 등록
     */
    public int save(BbsMngVO bbsMngVO) throws Exception;

    /**
     * @author  하석형
     * @since   2024.05.10
     * 게시판 관리 목록 조회 개수
     */
    public int findAllCnt(Pagination pagination) throws Exception;

    /**
     * @author  하석형
     * @since   2024.05.10
     * 게시판 관리 목록 조회
     */
    public List<BbsMngVO> findAll(Pagination pagination) throws Exception;

    /**
     * @author  박정하
     * @since   2024.05.16
     * 게시판 관리 목록 조회 (메뉴 관리용)
     */
    public List<BbsMngVO> findAllByMenuMng() throws Exception;

    /**
     * @author 하석형
     * @since  2024.05.08
     *
     * 게시판 관리 상세 조회
     */
    public BbsMngVO findByBbsMngId(String bbsMngId) throws Exception;

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 관리 수정
     */
    public int update(BbsMngVO bbsMngVO) throws Exception;

    /**
     * @author 하석형
     * @since  2024.05.09
     *
     * 게시판 관리 삭제
     */
    public int delete(BbsMngVO bbsMngVO) throws Exception;



    /** ********************************* 게시판 라우터 (시작) ********************************* */
    /**
     * @author 하석형
     * @since  2024.05.13
     *
     * 게시판 라우터 등록
     */
    public int saveBbsRouter(BbsRouterVO bbsRouterVO) throws Exception;

    /**
     * @author 하석형
     * @since  2024.05.13
     *
     * 게시판 라우터 수정
     */
    public int updateBbsRouter(BbsRouterVO bbsRouterVO) throws Exception;

    /**
     * @author 하석형
     * @since  2024.05.13
     *
     * 게시판 라우터 삭제
     */
    public int deleteBbsRouter(String bbsMngId) throws Exception;

    /**
     * @author 방선주
     * @since  2024.06.04
     *
     * 게시판 라우터 목록 조회
     */
    public List<BbsRouterVO> findAllBbsRouter(String bbsMngId);

    /**
     * @author 방선주
     * @since  2024.06.05
     *
     * 게시판 라우터 pshrcrs 수정
     */
    public void updateCrs(BbsRouterVO route);

    /** ********************************* 게시판 라우터 (종료) ********************************* */

    /**
     * @author  박정하
     * @since   2024.07.18
     * 게시판 관리 목록 조회 개수 (접속 통계용)
     */
    public int findAllCntNotPagination() throws Exception;
}