package com.takensoft.cms.bbs.service.Impl;

import com.takensoft.cms.bbs.dao.WordMngDAO;
import com.takensoft.cms.bbs.service.WordMngService;
import com.takensoft.cms.bbs.vo.WordMngVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;

@Service("WordMngService")
@RequiredArgsConstructor
public class WordMngServiceImpl extends EgovAbstractServiceImpl implements WordMngService {

    private final WordMngDAO wordMngDAO;
    private final JWTUtil jwtUtil;

    /**
     * @author 방선주
     * @since  2024.05.31
     *
     * 금지어 추가
     */
    @Override
    public int saveWords(HashMap<String, Object> wordList) throws Exception {
        // 등록된 토큰에서 사용자 정보 조회
        String writer = jwtUtil.getWriter();

        List<String> wordsArr = (List<String>) wordList.get("wordsArr");
        // 금지어 등록
        for (String word : wordsArr){
            WordMngVO wordMngVO = new WordMngVO();
            wordMngVO.setCntrlNm(word);
            wordMngVO.setRgtr(writer);
            wordMngDAO.saveWords(wordMngVO);
        }

        return wordList.size();
    }

    /**
     * @author 방선주
     * @since  2024.05.31
     *
     * 금지어 목록 조회
     */
    @Override
    public HashMap<String, Object> findAllWord(HashMap<String, String> params) throws Exception {
        HashMap<String, Object> result = new HashMap<>();

        Pagination search = new Pagination(0, params);
        int cnt = wordMngDAO.findAllCnt(search);
        Pagination pagination = new Pagination(cnt, params);
        List<WordMngVO> wordList = wordMngDAO.findAllWord(pagination);

        result.put("wordList", wordList);
        result.put("pagination", pagination);
        result.put("wordsCnt", cnt);

        return result;
    }
    /**
     * @author 방선주
     * @since  2024.05.31
     *
     * 금지어 삭제
     */
    @Override
    public int deleteWords(HashMap<String, Object> params) throws Exception {
        List<Integer> wordsArr = (List<Integer>) params.get("wordsArr");

        // 금지어 id로 삭제
        for (int cntrlNmId : wordsArr){
            wordMngDAO.deleteWords(cntrlNmId);
        }

        return wordsArr.size();
    }
}
