package com.takensoft.cms.bbs.web;

import com.takensoft.cms.bbs.service.BbsTypeMngService;
import com.takensoft.cms.bbs.vo.BbsTypeMngVO;
import com.takensoft.common.util.ResponseData;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.nio.charset.Charset;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.05.08
 *
 * 게시판 유형 관리 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value="/admin/bbsTypeMng")
public class BbsTypeMngController {
    private final BbsTypeMngService bbsTypeMngService;

    /**
     * @author  박정하
     * @since   2024.05.08
     * @param   bbsTypeMngVO
     * @return
     * @throws  Exception
     *
     * 게시판 유형 관리 등록
     */
    @PostMapping("/insertBbsTypeMng.json")
    public ResponseEntity<?> insertBbsTypeMng(@RequestBody BbsTypeMngVO bbsTypeMngVO) throws Exception {
        // 게시판 관리 등록
        HashMap<String, Object> result = bbsTypeMngService.insertBbsTypeMng(bbsTypeMngVO);
        int insertResult = (int) result.get("result");

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(insertResult > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 등록 처리되었습니다.");
            responseData.setData(result);
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("등록에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author  하석형
     * @since   2024.05.13
     * @param   bbsTypeMngVO
     * @return
     * @throws  Exception
     *
     * 게시판 유형 관리 목록 조회
     */
    @PostMapping("/findAll.json")
    public ResponseEntity<?> findAll(@RequestBody BbsTypeMngVO bbsTypeMngVO) throws Exception{
        // 게시판 유형 관리 목록 조회
        Map<String, Object> result = bbsTypeMngService.findAllBbsTypeMng(bbsTypeMngVO);

        // 응답처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }
}
