package com.takensoft.cms.contsType.service.Impl;

import com.takensoft.cms.bbs.dao.BbsRouterDAO;
import com.takensoft.cms.contsType.dao.ContsTypeDAO;
import com.takensoft.cms.contsType.service.ContsTypeService;
import com.takensoft.cms.contsType.vo.ContsAuthrtVO;
import com.takensoft.cms.contsType.vo.ContsTypeVO;
import com.takensoft.cms.menu.dao.MenuDAO;
import com.takensoft.cms.menu.service.MenuService;
import com.takensoft.cms.menu.vo.MenuAuthorVO;
import com.takensoft.cms.menu.vo.MenuVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author  : takensoft
 * @since   : 2024.04.17
 *
 * 콘텐츠 유형 관리 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * ContsTypeService : 콘텐츠 유형 관리 관련 인터페이스 상속
 */
@Service("contsTypeService")
@RequiredArgsConstructor
public class ContsTypeServiceImpl extends EgovAbstractServiceImpl implements ContsTypeService {
    private final ContsTypeDAO contsTypeDAO;
    private final BbsRouterDAO bbsRouterDAO;
    private final IdgenService contsTypeIdgn;
    private final JWTUtil jwtUtil;
    private final MenuService menuService; // 메뉴 인터페이스
    private final MenuDAO menuDAO;

    /**
     * @author takensoft
     * @since 2024.04.17
     * 콘텐츠 유형 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public HashMap<String, Object> contsSave(ContsTypeVO contsTypeVO) throws Exception {
        HashMap<String, Object> result = new HashMap<>();

        // 콘텐츠 유형 아이디 생성
        String contsId = contsTypeIdgn.getNextStringId();
        contsTypeVO.setContsId(contsId);

        // 등록된 토큰에서 사용자 정보 조회
        contsTypeVO.setRgtr(jwtUtil.getWriter());

        // 콘텐츠 유형 등록
        int insertResult = contsTypeDAO.save(contsTypeVO);

        // 메뉴 등록
        if (contsTypeVO.getExpsrYn().equals("Y")) {
            MenuVO menuVO = new MenuVO();
            menuVO.setUpMenuId("MENU_000000000000018"); // 상위메뉴아이디(콘텐츠 관리 메뉴 아이디)
            menuVO.setMenuNm(contsTypeVO.getContsKornNm()); // 메뉴명
            menuVO.setMenuType("conts"); // 메뉴타입
            menuVO.setMenuTypeCtgry(contsId); // 메뉴타입항목(콘텐츠 유형 아이디)
            menuVO.setLinkType("0"); // 링크유형(현재창)
            menuVO.setDgstfnExmnUseYn("N"); // 만족도조사사용여부(미노출)
            menuVO.setMenuExpsrYn("Y"); // 메뉴노출여부(노출)
            menuVO.setMenuUser("A"); // 메뉴사용자(관리자)
            menuVO.setSysPvsnYn("0"); // 시스템제공여부(시스템등록)
            menuService.menuSave(menuVO);
        }
        
        result.put("contsId", contsId);
        result.put("result", insertResult);
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * 콘텐츠 유형 수정
     */
    @Override
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int contsUpdate(ContsTypeVO contsTypeVO) throws Exception {
        int result = 0;
        // 등록된 토큰에서 사용자 정보 조회
        contsTypeVO.setMdfr(jwtUtil.getWriter());
        result += contsTypeDAO.update(contsTypeVO);

        // 메뉴 수정
        menuDAO.updateByContsId(contsTypeVO);
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * 콘텐츠 유형 목록 조회
     */
    @Override
    public Map<String, Object> findAll(Map<String, String> params) throws Exception {
        Map<String, Object> result = new HashMap<>();
        Pagination search = new Pagination(0, params);
        int cnt = contsTypeDAO.findAllCnt(search);
        Pagination pagination = new Pagination(cnt, params);
        List<ContsTypeVO> list = contsTypeDAO.findAll(pagination);

        result.put("list", list);
        result.put("pagination", pagination);
        return result;
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * 콘텐츠 유형 목록 조회(노출인 목록만)
     */
    @Override
    public List<ContsTypeVO> findByContsTypeSys(ContsTypeVO contsTypeVO) throws Exception {
        return contsTypeDAO.findByContsTypeSys(contsTypeVO);
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * 콘텐츠 유형 상세 조회
     */
    @Override
    public ContsTypeVO findByContsType(String contsId) throws Exception {
        ContsTypeVO contsTypeVO = new ContsTypeVO();

        if (contsId != null) {
            contsTypeVO = contsTypeDAO.findByContsType(contsId);
        }

        // 메뉴노출 기본값
        if (contsTypeVO.getExpsrYn() == null) {
            contsTypeVO.setExpsrYn("N");
        }
        // 사용여부 기본값
        if (contsTypeVO.getUseYn() == null) {
            contsTypeVO.setUseYn("Y");
        }

        return contsTypeVO;
    }

    /**
     * @author 박정하
     * @since 2024.04.29
     * 콘텐츠 유형 목록 조회 (메뉴 노출 건만)
     */
    @Override
    public List<ContsTypeVO> findAllByExpsr(ContsTypeVO contsTypeVO) throws Exception {
        return contsTypeDAO.findAllByExpsr(contsTypeVO);
    }

    /**
     * @author takensoft
     * @since 2024.05.02
     * 콘텐츠 조회(권한 포함)
     */
    @Override
    @Cacheable(value = "routerAuthInfo", key = "'routerAuth'")
    public List<ContsAuthrtVO> findByContsAuthrt() throws Exception {
        // 콘텐츠(라우터)정보 조회
        List<ContsAuthrtVO> list = contsTypeDAO.findByContsAuthrt();
        // 권한 공유
        List<ContsAuthrtVO> result = groupAndShareAuth(list);

        // 게시판 라우터 정보 조회
        List<ContsAuthrtVO> bbsList = bbsRouterDAO.findByBbsRouterAuthrt();
        result.addAll(bbsList);

        return result;
    }

    /**
     * @author toakensoft
     * @since 2024.05.08
     * 라우터의 특정경로로 그룹화하여 각 그룹 내에서 권한을 공유
     */
    private List<ContsAuthrtVO> groupAndShareAuth(List<ContsAuthrtVO> routes) {
        // 경로의 특정 기준점으로 라우터를 그룹화
        Map<String, List<ContsAuthrtVO>> groupedRoutes = routes.stream()
                .collect(Collectors.groupingBy(route -> route.getPageCrs().substring(0, route.getPageCrs().lastIndexOf('/')))); // 마지막 슬래시까지 포함된 경로 추출

        /* key */
//        Iterator<String> keys = groupedRoutes.keySet().iterator();
//        while(keys.hasNext()) {
//            String key = keys.next();
//            System.out.println("key : " + key);
//        }
        /* key */

        // 그룹화된 각 경로에 대해 권한 정보 공유
        groupedRoutes.forEach((basePath, routeGroup) -> {
            List<MenuAuthorVO> sharedAuth = findMostComprehensiveAuth(routeGroup); // 그룹 내에서 가장 완전한 권한 정보 조회
            // 그룹의 모든 라우트에 권한 정보가 없으면 공유된 권한 정보로 설정
            routeGroup.forEach(route -> {
                if(route.getAuthrtList().isEmpty()) {
                    route.setAuthrtList(sharedAuth);
                }
            });
        });
        // 각 그룹의 라우터들을 리스트로 반환
        return groupedRoutes.values().stream()
                .flatMap(List::stream)
                .collect(Collectors.toList());
    }

    /**
     * @author toakensoft
     * @since 2024.05.08
     * 그룹 내에서 가장 완전한 권한 정보를 가진 라우터를 찾아 해당 권한 리트스 반환
     */
    private List<MenuAuthorVO> findMostComprehensiveAuth(List<ContsAuthrtVO> routeGroup) {
        // 권한정보가 있는 첫 번째 라우터를 찾아 권한 정보 반환
        return routeGroup.stream()
                .filter(route -> !route.getAuthrtList().isEmpty())
                .findFirst()
                .map(ContsAuthrtVO::getAuthrtList)
                .orElse(Collections.emptyList()); // 권한 정보가 없으면 빈 리스트 반환
    }
}
