package com.takensoft.cms.contsType.web;

import com.takensoft.cms.contsType.service.ContsTypeService;
import com.takensoft.cms.contsType.vo.ContsTypeVO;
import com.takensoft.common.util.ResponseData;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.nio.charset.Charset;
import java.util.*;

/**
 * @author  : takensoft
 * @since   : 2024.04.17
 *
 * 콘텐츠 유형 관리 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/admin/contsType")
public class ContsTypeController {
    private final ContsTypeService contsTypeService;

    /**
     * @author takensoft
     * @since 2024.04.17
     * @param contsTypeVO
     * @return
     * @throws Exception
     *
     * 콘텐츠 유형 등록
     */
    @PostMapping("/saveProc.json")
    public ResponseEntity<?> savePorc(@RequestBody ContsTypeVO contsTypeVO) throws Exception {
        // 콘텐츠 유형 등록
        HashMap<String, Object> result = contsTypeService.contsSave(contsTypeVO);
        int insertResult = (int) result.get("result");

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(insertResult > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 등록 처리되었습니다.");
            responseData.setData(result);
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("등록에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * @param contsTypeVO
     * @return
     * @throws Exception
     *
     * 콘텐츠 유형 수정
     */
    @PostMapping("/updateProc.json")
    public ResponseEntity<?> updateProc(@RequestBody ContsTypeVO contsTypeVO) throws Exception {
        // 콘텐츠 유형 수정 처리
        int result = contsTypeService.contsUpdate(contsTypeVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 수정 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("수정에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * @param params
     * @return
     * @throws Exception
     *
     * 콘텐츠 유형 목록 조회
     */
    @PostMapping("/findAll.json")
    public ResponseEntity<?> findAll(@RequestBody Map<String, String> params) throws Exception {
        // 콘텐츠 유형 목록 조회
        Map<String, Object> result = contsTypeService.findAll(params);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * @param contsTypeVO
     * @return
     * @throws Exception
     *
     * 콘텐츠 유형 상세 조회
     */
    @PostMapping("/findByContsType.json")
    public ResponseEntity<?> findByContsType(@RequestBody ContsTypeVO contsTypeVO) throws Exception {
        ContsTypeVO result = contsTypeService.findByContsType(contsTypeVO.getContsId());

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author takensoft
     * @since 2024.04.17
     * @param contsTypeVO
     * @return
     * @throws Exception
     *
     * 콘텐츠 유형 삭제
     */
    @PostMapping("/deleteProc.json")
    public ResponseEntity<?> deleteProc(@RequestBody ContsTypeVO contsTypeVO) throws Exception {
        contsTypeVO.setUseYn("N");
        int result = contsTypeService.contsUpdate(contsTypeVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 삭제 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("삭제에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author 박정하
     * @since 2024.04.29
     * @param
     * @return
     * @throws Exception
     *
     * 콘텐츠 유형 목록 조회 (메뉴 노출 건만)
     */
    @PostMapping("/findAllByExpsr.json")
    public ResponseEntity<?> findAllByExpsr(@RequestBody ContsTypeVO contsTypeVO) throws Exception {
        // 콘텐츠 유형 목록 조회 (메뉴 노출 건만)
        List<ContsTypeVO> result = contsTypeService.findAllByExpsr(contsTypeVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }
}
