package com.takensoft.cms.dept.dao;

import com.takensoft.cms.dept.vo.DeptAuthrtVO;
import com.takensoft.cms.dept.vo.DeptMbrVO;
import com.takensoft.cms.dept.vo.DeptVO;
import com.takensoft.common.HierachyVO;
import org.egovframe.rte.psl.dataaccess.mapper.Mapper;

import java.util.List;
import java.util.Map;

/**
 * @author  : takensoft
 * @since   : 2024.04.24
 *
 * 부서 정보 관련 Mapper
 */
@Mapper("deptDAO")
public interface DeptDAO {
    /**
     * @author takensoft
     * @since 2024.04.25
     * 부서 등록
     */
    int save(DeptVO deptVO);

    /**
     * @author takensoft
     * @since 2024.04.25
     * 부서 권한 등록
     */
    int authrtSave(DeptAuthrtVO deptAuthrtVO);

    /**
     * @author takensoft
     * @since 2024.04.29
     * 부서 사용자 등록
     */
    int deptMbrSave(DeptMbrVO deptMbrVO);

    /**
     * @author takensoft
     * @since 2024.04.25
     * 부서 깊이(레벨) 조회
     */
    int findByDeptGrd(String upDeptId);

    /**
     * @author takensoft
     * @since 2024.04.25
     * 부서 순서 조회
     */
    int findByDeptSn(String upDeptId);

    /************************************** Hierachy 전용 **************************************/
    /**
     * @author takensoft
     * @since 2024.04.25
     * 최상위 부서 조회
     */
    List<HierachyVO> findByTopNode();
    /**
     * @author takensoft
     * @since 2024.04.24
     * 하위 부서 조회
     */
    List<HierachyVO> findChildNode(String id);
    /************************************** Hierachy 전용 **************************************/

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 정보 조회
     */
    DeptVO findByDept(String deptId);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 정보 수정
     */
    int update(DeptVO deptVO);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 권한 삭제
     */
    int deleteAuth(String deptId);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서 사용자 등록을 위한 사용자 정보 목록 조회 (부서에 등록되지 않은 사용자 목록)
     */
    List<DeptMbrVO> findByMber(Map<String, String> params);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서에 등록된 사용자 목록 조회
     */
    List<DeptMbrVO> findByDeptMber(String deptId);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 상위,하위 부서 삭제
     */
    int deleteDept(String deptId);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 상위, 하위 부서 삭제에 따른 부서의 사용자 삭제
     */
    int deleteDeptInDeptMbr(String deptId);

    /**
     * @author takensoft
     * @since 2024.04.26
     * 부서에 등록된 사용자 제거
     */
    int deleteDeptMbr(Map<String, Object> params);

    /**
     * @author 박정하
     * @since 2024.05.10
     * 부서 정보 수정 (hierachyVO 사용)
     */
    public int deptUpdateByHierachy(DeptVO deptVO) throws Exception;
}