package com.takensoft.cms.main.web;

import com.takensoft.cms.main.service.MainService;
import com.takensoft.common.util.ResponseData;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.CacheManager;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.nio.charset.Charset;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.05.29
 *
 * 메인 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/admin/main")
public class MainController {
    private final MainService mainService;
    private final CacheManager cacheManager; // 캐시를 관리하기 위한 CacheManager 인스턴스

    /**
     * @author takensoft
     * @since 2024.05.27
     * @param
     * @return
     * @throws Exception
     *
     * 관리자 메인페이지 통계정보 조회 [ 신규회원 가입자 수, 오늘 방문자 수, 이번달 방문자 수, 총 방문자 수, 방문자 접속 통계 ]
     */
    @PostMapping("/adminMainProc.json")
    public ResponseEntity<?> adminMainProc() throws Exception {
        // 관리자 메인페이지 통계정보 조회
        Map<String, Object> result = mainService.selectAdminMain();

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author takensoft
     * @since 2024.05.29
     * @param
     * @return
     * @throws Exception
     *
     * 캐시 초기화
     */
    @PostMapping("/cacheReSet.json")
    public ResponseEntity<?> serverRestart() throws Exception {
        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();

        try {
            for(String nm : cacheManager.getCacheNames()) {
                cacheManager.getCache(nm).clear();
            }
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } catch (Exception e) {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("적용에 실패했습니다.\n관리자에게 문의해주시기 바랍니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        }
    }

    /**
     * @author 박정하
     * @since 2024.06.14
     * @param
     * @return
     * @throws Exception
     *
     * 관리자 메인페이지 기업상담신청 알람
     */
    @PostMapping("/findAplyToastsProc.json")
    public ResponseEntity<?> findAplyToastsProc() throws Exception {
        List<HashMap<String, Object>> result = mainService.findAplyToasts();

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }
}