package com.takensoft.cms.menu.service.Impl;

import com.takensoft.cms.author.dao.AuthorDAO;
import com.takensoft.cms.author.vo.AuthorVO;
import com.takensoft.cms.menu.dao.MenuAuthorDAO;
import com.takensoft.cms.menu.dao.MenuDAO;
import com.takensoft.cms.menu.service.MenuAuthorService;
import com.takensoft.cms.menu.vo.MenuAuthorVO;
import com.takensoft.cms.menu.vo.MenuVO;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.05.07
 *
 * 메뉴 권한 정보 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * MenuAuthorService : 메뉴 정보 인터페이스 상속
 */
@Service("menuAuthorServiceImpl")
@RequiredArgsConstructor
public class MenuAuthorServiceImpl extends EgovAbstractServiceImpl implements MenuAuthorService {
    private final MenuDAO menuDAO;
    private final MenuAuthorDAO menuAuthorDAO;
    private final AuthorDAO authorDAO;
    private final JWTUtil jwtUtil;

    /**
     * @author takensoft
     * @since 2024.04.15
     * 메뉴별 권한 등록을 위한 전처리
     */
    private MenuAuthorVO saveMenuAuthor(String menuId, String AuthrtCd) {
        MenuAuthorVO menuAuthorVO = new MenuAuthorVO();
        menuAuthorVO.setMenuId(menuId);
        menuAuthorVO.setAuthrtCd(AuthrtCd);
        return menuAuthorVO;
    }

    /**
     * @author takensoft
     * @since 2024.04.15
     * 메뉴별 권한 등록
     * 파라미터로 들어오는 id는 type에 따라 권한 코드 or 메뉴 아이디로 들어옴
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int menuAuthrtSave(String id, String type) throws Exception {
        int result = 0;
        // 타입이 null or empty 경우 에러 발생
        if(type == null || type.equals("")) {
            throw new IllegalArgumentException("type is null or empty");
        }
        // 신규 메뉴 등록 시
        if("menu".equals(type)) {
            // 등록된 권한 목록 조회
            List<AuthorVO> authList = authorDAO.findAllSystem();
            for(AuthorVO author : authList) {
                MenuAuthorVO menuAuthor = saveMenuAuthor(id, author.getAuthrtCd());
                result += menuAuthorDAO.menuAuthrtSave(menuAuthor);
            }
            // 신규 권한 등록 시
        } else if("author".equals(type)) {
            // 등록된 메뉴 목록 조회
            List<MenuVO> menuList = menuDAO.findExceptTopMenu();
            for(MenuVO menu : menuList) {
                MenuAuthorVO menuAuthor = saveMenuAuthor(menu.getMenuId(), id);
                result += menuAuthorDAO.menuAuthrtSave(menuAuthor);
            }
            // 타입이 맞지 않다면 에러
        } else {
            throw new IllegalArgumentException("Invalid value : " + type);
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.05.02
     * 메뉴 권한 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = "routerAuthInfo", allEntries = true)
    public int updateMenuAuthrtProc(HashMap<String, Object> params) throws Exception {
        int result = 0;
        List<HashMap<String, Object>> menuList = (List<HashMap<String, Object>>)params.get("menuList");
        String authrtCd = params.get("authrtCd").toString();
        for(HashMap<String, Object> menu : menuList) {
            menu.put("mdfr", jwtUtil.getWriter());
            menu.put("authrtCd", authrtCd);
            result += menuAuthorDAO.updateMenuAuthrt(menu);
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.05.08
     * 메뉴 권한 조회
     */
    @Override
    public List<HashMap<String, Object>> findAll(AuthorVO authorVO) throws Exception {
        List<HashMap<String, Object>> result = menuAuthorDAO.findAll(authorVO);
        for(HashMap<String, Object> menuAuthor : result) {
            if (menuAuthor.get("upMenuId") != null) {
                if (menuAuthor.get("inqAuthrt") != null && menuAuthor.get("regAuthrt") != null && menuAuthor.get("mdfcnAuthrt") != null && menuAuthor.get("delAuthrt") != null && menuAuthor.get("fileDwnldAuthrt") != null) {
                    if(menuAuthor.get("inqAuthrt").equals("Y") && menuAuthor.get("regAuthrt").equals("Y") && menuAuthor.get("mdfcnAuthrt").equals("Y") && menuAuthor.get("delAuthrt").equals("Y") && menuAuthor.get("fileDwnldAuthrt").equals("Y")) {
                        menuAuthor.put("checkedAll", "Y");
                    } else {
                        menuAuthor.put("checkedAll", "N");
                    }
                }
            }
        }
        return result;
    }
}