package com.takensoft.cms.menu.web;

import com.takensoft.cms.author.vo.AuthorVO;
import com.takensoft.cms.menu.service.MenuAuthorService;
import com.takensoft.common.util.ResponseData;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.List;

/**
 * @author  : 박정하
 * @since   : 2024.05.07
 *
 * 메뉴 권한 정보 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/admin/menuAuthor")
public class MenuAuthorController {
    private final MenuAuthorService menuAuthorService;

    /**
     * @author 박정하
     * @since 2024.05.02
     * @param
     * @return
     * @throws Exception
     *
     * 메뉴 권한 목록 조회
     */
    @PostMapping(value = "/findAllByAuthor.json")
    public ResponseEntity<?> findAllByAuthor(@RequestBody AuthorVO authorVO) throws Exception {
        List<HashMap<String, Object>> result = menuAuthorService.findAll(authorVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 메뉴 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.05.02
     * @param params
     * @return
     * @throws Exception
     *
     * 메뉴 권한 수정
     */
    @PostMapping(value = "/updateMenuAuthrtProc.json")
    public ResponseEntity<?> updateMenuAuthrtProc(@RequestBody HashMap<String, Object> params) throws Exception {
        // 메뉴 권한 수정
        int result = menuAuthorService.updateMenuAuthrtProc(params);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 메뉴 권한 수정 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("메뉴 권한 수정에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
