package com.takensoft.cms.search.service.Impl;

import com.takensoft.cms.menu.service.MenuService;
import com.takensoft.cms.menu.vo.MenuRequest;
import com.takensoft.cms.menu.vo.SysMenuVO;
import com.takensoft.cms.search.dao.SearchDAO;
import com.takensoft.cms.search.service.SearchService;
import com.takensoft.cms.search.vo.SearchContentVO;
import com.takensoft.cms.search.vo.SearchMenuVO;
import com.takensoft.cms.search.vo.SearchResultVO;
import com.takensoft.cms.search.vo.SearchVO;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringEscapeUtils;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author  : 하석형
 * @since   : 2024.06.12
 *
 * 통합 검색 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * SearchService : 통합 검색 인터페이스 상속
 */
@Service("searchService")
@RequiredArgsConstructor
public class SearchServiceImpl extends EgovAbstractServiceImpl implements SearchService {

    private final MenuService menuService;
    private final SearchDAO searchDAO;

    /**
     * @author  하석형
     * @since   2024.06.12
     * 통합 검색
     */
    public SearchResultVO searchAll (SearchVO searchVO) throws Exception {
        SearchResultVO searchResultVO = new SearchResultVO();

        /* 메뉴 목록 */
        MenuRequest mReq = new MenuRequest();
        mReq.setMenuType(searchVO.getMenuType()); // 메뉴타입: "government"
        mReq.setRoles(searchVO.getRoles()); // 로그인 사용자 권한: "ROLE_ADMIN" / "ROLE_USER"
        SysMenuVO topMenu = new SysMenuVO();
        List<SysMenuVO> menuList = menuService.findByMenuWithRouter(mReq);
        topMenu.setChildList(menuList);

        //TODO: 메뉴명 검색

        /* 검색 */
        this.searchMenu(topMenu, searchVO, searchResultVO);

        return searchResultVO;
    }

    /**
     * @author  하석형
     * @since   2024.06.12
     * 검색 함수
     */
    public void searchMenu (SysMenuVO sysMenu, SearchVO searchVO, SearchResultVO searchResultVO) throws Exception {
        List<SysMenuVO> menuList = sysMenu.getChildList();
        for(SysMenuVO menu : menuList) {
            SearchMenuVO searchMenu = new SearchMenuVO(); // 검색 메뉴 정보
            searchMenu.setMenuId(menu.getMenuId());
            searchMenu.setRouterUrl(menu.getRouterUrl());

            // 전체 메뉴명 생성
            String fullMenuNm = "";
            if(sysMenu.getMenuNm() != null && !sysMenu.getMenuNm().isEmpty()) {
                fullMenuNm =  sysMenu.getMenuNm() + " > ";
            }
            fullMenuNm = fullMenuNm + menu.getMenuNm();
            searchMenu.setFullMenuNm(fullMenuNm);

            // 자식 메뉴 존재 시 재귀 처리
            if (!menu.getChildList().isEmpty()) {
                menu.setMenuNm(fullMenuNm); // 자식 메뉴에 사용할 이전 전체 메뉴명
                this.searchMenu(menu, searchVO, searchResultVO);
            } else {
                // 메뉴 타입이 있을 시
                if (menu.getMenuType() != null && !menu.getMenuType().isEmpty()) {
                    String menuType = menu.getMenuType(); // bbs: 게시판, conts: 컨텐츠
                    // 메뉴 URL이 있을 시
//                    if(menu.getRouterUrl() != null && !menu.getRouterUrl().isEmpty()) {
//                        String routerUrl = menu.getRouterUrl(); // 메뉴 URL
//                        if (routerUrl.contains(".page")) {
//                            int lastSlashIndex = routerUrl.lastIndexOf('/'); // 마지막 '/' 인덱스
//                            String path = routerUrl.substring(0, lastSlashIndex); // 마지막 '/' 이전 경로
//                        }
//                    }

                    /* 게시판 검색 */
                    if (menuType.equals("bbs")) {
                        // 타입이 게시판일 시 게시판 내용 테이블에서 조회
                        String bbsMngId = menu.getMenuTypeCtgry(); // 게시판 관리 ID
                        searchVO.setBbsMngId(bbsMngId);

                        //TODO: (사용중)방법1.전체 개수 구하는 SQL, 미리보기 개수만큼 목록을 조회하는 SQL
                        //TODO: 방법2.전체 목록을 검색해서 개수를 구한 뒤, 미리보기 개수만큼 for문 사용해 목록 상단에서부터 따로 담기

                        // 전체 검색 목록 개수
                        int totalListCount = searchDAO.findAllCountByBbs(searchVO);
                        searchResultVO.setPageCount(searchResultVO.getPageCount() + totalListCount); // 페이지 개수
                        searchResultVO.setBoardCount(searchResultVO.getBoardCount() + totalListCount); // 게시판 개수
                        searchMenu.setTotalListCount(totalListCount);

                        // 미리보기 검색 게시판 목록
                        List<SearchContentVO> bbsList = searchDAO.findAllByBbs(searchVO);
                        for(SearchContentVO bbs : bbsList) {
                            bbs.setContent(this.stripHtmlTags(bbs.getContent())); // HTML 태그 제거
                        }
                        if (!bbsList.isEmpty()) {
                            searchMenu.setPreviewList(bbsList); // 미리보기 목록 추가
                            searchResultVO.getPageList().add(searchMenu); // 페이지 목록에 추가
                            searchResultVO.getBoardList().add(searchMenu); // 게시판 목록에 추가
                        }
                    }
                    /* 컨텐츠 검색 */
                    else if (menuType.equals("conts")) {
                        // 타입이 콘텐츠일 시 하드코딩으로 검색 조회
                        String contsId = menu.getMenuTypeCtgry(); // 컨텐츠 ID
                        List<SearchContentVO> contentList = new ArrayList<>();

                        // 기업 정보 검색
                        if(contsId.equals("CONTS_000000000000030")) {
                            this.searchEntInfo(searchVO, contentList, searchMenu, searchResultVO);
                        }
                        // 기업 상담 신청 검색
                        else if(contsId.equals("CONTS_000000000000035")){
                            this.searchEntDscsnAply(searchVO, contentList, searchMenu, searchResultVO);
                        }
                        // 투자 상담 검색
                        else if(contsId.equals("CONTS_000000000000040")){
                            this.searchIvstDscsn(searchVO, contentList, searchMenu, searchResultVO);
                        }
                        // 검토 사항 검색
                        else if(contsId.equals("CONTS_000000000000037")){
                            this.searchRvwMttr(searchVO, contentList, searchMenu, searchResultVO);
                        }

                        if (!contentList.isEmpty()) {
                            searchMenu.setPreviewList(contentList); // 미리보기 목록 추가
                            searchResultVO.getPageList().add(searchMenu); // 페이지 목록에 추가
                            searchResultVO.getContentList().add(searchMenu); // 컨텐츠 목록에 추가
                        }
                    }
                }
            }
        }
    }

    /**
     * @author  하석형
     * @since   2024.06.13
     * HTML 태그 제거
     */
    public String stripHtmlTags(String html) {
        if (html == null || html.isEmpty()) {
            return html;
        }
        // HTML 태그 제거
        String noTags = html.replaceAll("<[^>]*>", "");
        // HTML 엔티티 디코딩
        return StringEscapeUtils.unescapeHtml4(noTags);
    }



    /** ********************************* 컨텐츠 검색 - 기업 정보 (시작) ********************************* */
    /**
     * @author  하석형
     * @since   2024.06.14
     * 기업 정보 검색
     */
    public void searchEntInfo (SearchVO searchVO, List<SearchContentVO> contentList, SearchMenuVO searchMenu, SearchResultVO searchResultVO) throws Exception {
        // 전체 검색 목록 개수
        int totalListCount = searchDAO.findAllCountByEntInfo(searchVO);
        this.searchListCount(totalListCount, searchMenu, searchResultVO);

        // 미리보기 검색 컨텐츠 목록
        List<HashMap<String, Object>> contsLists = searchDAO.findAllByEntInfo(searchVO);
        for(HashMap<String, Object> conts : contsLists) {
            String entNm = conts.get("ent_nm").toString(); // 기업명
            String rprsvNm = conts.get("rprsv_nm").toString(); // 대표자명
//                                String clsbizYn = ""; // 기업상태
//                                if(conts.get("clsbiz_yn").equals("Y")) {
//                                    clsbizYn = "폐업";
//                                } else if(conts.get("clsbiz_yn").equals("N")) {
//                                    clsbizYn = "정상";
//                                } else {
//                                    clsbizYn = "-";
//                                }
//                                conts.get("ivst_dscsn_inten"); // (보류)입주의향
            String rmrk = Optional.ofNullable(conts.get("rmrk")).map(Object::toString).orElse(""); // 비고
            String regDt = conts.get("reg_dt").toString(); // 등록일
            String pageId = conts.get("page_id").toString(); // 페이지 ID

            String makeTitle = String.format("[기업명: %s] [대표자: %s]", entNm, rprsvNm); // 제목
            String makeContent =  rmrk; // 내용

            // 검색 결과 형태로 변환
            this.convertToSearchContent(contentList, makeTitle, makeContent, regDt, pageId);
        }
    }

    /**
     * @author  하석형
     * @since   2024.06.14
     * 기업 상담 신청 검색
     */
    public void searchEntDscsnAply (SearchVO searchVO, List<SearchContentVO> contentList, SearchMenuVO searchMenu, SearchResultVO searchResultVO) throws Exception {
        // 전체 검색 목록 개수
        int totalListCount = searchDAO.findAllCountByEntDscsnAply(searchVO);
        this.searchListCount(totalListCount, searchMenu, searchResultVO);

        // 미리보기 검색 컨텐츠 목록
        List<HashMap<String, Object>> contsLists = searchDAO.findAllByEntDscsnAply(searchVO);
        for(HashMap<String, Object> conts : contsLists) {
            String entNm = conts.get("ent_nm").toString(); // 기업명
            String prcsStts = conts.get("prcs_stts").toString(); // 진행상태
            String rmrk = Optional.ofNullable(conts.get("rmrk")).map(Object::toString).orElse(""); // 비고
            String regDt = conts.get("reg_dt").toString(); // 등록일
            String pageId = conts.get("page_id").toString(); // 페이지 ID

            String makeTitle = String.format("(%s)[기업명: %s]", prcsStts, entNm); // 제목
            String makeContent =  rmrk; // 내용

            // 검색 결과 형태로 변환
            this.convertToSearchContent(contentList, makeTitle, makeContent, regDt, pageId);
        }
    }

    /**
     * @author  하석형
     * @since   2024.06.14
     * 투자 상담 검색
     */
    public void searchIvstDscsn (SearchVO searchVO, List<SearchContentVO> contentList, SearchMenuVO searchMenu, SearchResultVO searchResultVO) throws Exception {
        // 전체 검색 목록 개수
        int totalListCount = searchDAO.findAllCountByIvstDscsn(searchVO);
        this.searchListCount(totalListCount, searchMenu, searchResultVO);

        // 미리보기 검색 컨텐츠 목록
        List<HashMap<String, Object>> contsLists = searchDAO.findAllByIvstDscsn(searchVO);
        for(HashMap<String, Object> conts : contsLists) {
            String entNm = conts.get("ent_nm").toString(); // 기업명
            String ttl = conts.get("ttl").toString(); // 제목
            String mainCn = Optional.ofNullable(conts.get("main_cn")).map(Object::toString).orElse(""); // 주요내용
            String regDt = conts.get("reg_dt").toString(); // 등록일
            String pageId = conts.get("page_id").toString(); // 페이지 ID

            String makeTitle = String.format("[기업명: %s] - %s", entNm, ttl); // 제목
            String makeContent =  mainCn; // 내용

            // 검색 결과 형태로 변환
            this.convertToSearchContent(contentList, makeTitle, makeContent, regDt, pageId);
        }
    }

    /**
     * @author  하석형
     * @since   2024.06.14
     * 검토 사항 검색
     */
    public void searchRvwMttr (SearchVO searchVO, List<SearchContentVO> contentList, SearchMenuVO searchMenu, SearchResultVO searchResultVO) throws Exception {
        // 전체 검색 목록 개수
        int totalListCount = searchDAO.findAllCountByRvwMttr(searchVO);
        this.searchListCount(totalListCount, searchMenu, searchResultVO);

        // 미리보기 검색 컨텐츠 목록
        List<HashMap<String, Object>> contsLists = searchDAO.findAllByRvwMttr(searchVO);
        for(HashMap<String, Object> conts : contsLists) {
            String entNm = conts.get("ent_nm").toString(); // 기업명
            String ttl = conts.get("ttl").toString(); // 투자상담명
            String type = conts.get("type").toString(); // 유형
            String mainCn = Optional.ofNullable(conts.get("rcpt_cn")).map(Object::toString).orElse(""); // 접수내용
            String regDt = conts.get("reg_dt").toString(); // 등록일
            String pageId = conts.get("page_id").toString(); // 페이지 ID

            String makeTitle = String.format("(%s)[기업명: %s] - %s", type, entNm, ttl); // 제목
            String makeContent =  mainCn; // 내용

            // 검색 결과 형태로 변환
            this.convertToSearchContent(contentList, makeTitle, makeContent, regDt, pageId);
        }
    }

    /**
     * @author  하석형
     * @since   2024.06.14
     * 전체 검색 목록 조회 개수
     */
    public void searchListCount (int totalListCount, SearchMenuVO searchMenu, SearchResultVO searchResultVO) throws Exception {
        searchResultVO.setPageCount(searchResultVO.getPageCount() + totalListCount); // 페이지 개수
        searchResultVO.setContentCount(searchResultVO.getContentCount() + totalListCount); // 게시판 개수
        searchMenu.setTotalListCount(totalListCount);
    }

    /**
     * @author  하석형
     * @since   2024.06.14
     * 컨텐츠를 검색 결과 형태로 변환
     */
    public void convertToSearchContent (List<SearchContentVO> contentList, String title, String content, String regDt, String pageId) throws Exception {
        SearchContentVO searchContentVO = new SearchContentVO();
        searchContentVO.setTitle(title);
        searchContentVO.setContent(content);
        searchContentVO.setRegDt(regDt);
        searchContentVO.setPageId(pageId);
        contentList.add(searchContentVO);
    }
    /** ********************************* 컨텐츠 검색 - 기업 정보 (종료) ********************************* */
}