package com.takensoft.common.excel.service.Impl;

import com.takensoft.cms.cntnStats.dto.CntnStatsExcelDTO;
import com.takensoft.common.excel.resource.ExcelRenderResource;
import com.takensoft.common.excel.resource.ExcelRenderResourceFactory;
import com.takensoft.common.excel.service.ExcelCreateService;
import com.takensoft.common.excel.service.ExcelService;
import com.takensoft.portal.entInfo.dto.BplcExcelDTO;
import com.takensoft.portal.entInfo.dto.EntInfoExcelDTO;
import com.takensoft.portal.ivstDscsn.dto.IvstDscsnExcelDTO;
import com.takensoft.portal.rvwMttr.dto.RvwMttrExcelDTO;
import com.takensoft.portal.rvwMttr.dto.RvwMttrPrgrsExcelDTO;
import lombok.RequiredArgsConstructor;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;

@Service("excelService")
@RequiredArgsConstructor
public class ExcelServiceImpl extends EgovAbstractServiceImpl implements ExcelService {
    // 엑셀 파일 출력
    public static void writeExcel(HttpServletResponse response, Workbook workbook, ExcelRenderResource resource) throws IOException {
        String fileName = new String(resource.getExcelFileName().getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
        response.setContentType("ms-vnd/excel");
        response.setHeader(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=" + fileName + ".xlsx"));
        workbook.write(response.getOutputStream());
        workbook.close();
    }

    // 투자상담
    @Override
    public void ivstDscsnExcelDownload(HttpServletResponse response, IvstDscsnExcelDTO ivstDscsn) throws Exception {
        // 기본 세팅
        Workbook workbook = new SXSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        int rowIndex = 0;

        // 개별 설정
        ExcelRenderResource mainResource = ExcelRenderResourceFactory.prepareRenderResource(IvstDscsnExcelDTO.class);
        List<IvstDscsnExcelDTO> mainDataList = new ArrayList<>();
        mainDataList.add(ivstDscsn);

        // 엑셀 그리기
        ExcelCreateService<IvstDscsnExcelDTO> mainExcel = new ExcelCreateService(workbook, sheet, mainDataList, mainDataList.getClass());
        mainExcel.createVertical(mainResource, mainDataList, rowIndex);

        // 엑셀 사이즈 지정
        sheetSize(sheet, 2);

        // 엑셀 파일 출력
        writeExcel(response, workbook, mainResource);
    }

    // 검토사항
    @Override
    public void rvwMttrExcelDownload(HttpServletResponse response, RvwMttrExcelDTO rvwMttr, List<RvwMttrPrgrsExcelDTO> prgrsList) throws Exception {
        // 기본 세팅
        Workbook workbook = new SXSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        int rowIndex = 0;

        // 개별 설정
        ExcelRenderResource mainResource = ExcelRenderResourceFactory.prepareRenderResource(RvwMttrExcelDTO.class);
        List<RvwMttrExcelDTO> mainDataList = new ArrayList<>();
        mainDataList.add(rvwMttr);

        ExcelRenderResource prgrsResource = ExcelRenderResourceFactory.prepareRenderResource(RvwMttrPrgrsExcelDTO.class);

        // 엑셀 그리기
        ExcelCreateService<RvwMttrExcelDTO> mainExcel = new ExcelCreateService(workbook, sheet, mainDataList, mainDataList.getClass());
        rowIndex = mainExcel.createCaption(rowIndex, "기본정보");
        rowIndex = mainExcel.createVertical(mainResource, mainDataList, rowIndex);

        rowIndex = mainExcel.createHr(rowIndex); // 구분선
        rowIndex = mainExcel.createCaption(rowIndex, "진행단계 및 지원방안");
        ExcelCreateService<RvwMttrPrgrsExcelDTO> prgrsExcel = new ExcelCreateService(workbook, sheet, prgrsList, prgrsList.getClass());
        rowIndex = prgrsExcel.createVH(prgrsResource, prgrsList, rowIndex);

        // 엑셀 사이즈 지정
        sheetSize(sheet, 2);

        // 엑셀 파일 출력
        writeExcel(response, workbook, mainResource);
    }

    // 기업정보
    @Override
    public void entInfoExcelDownload(HttpServletResponse response, EntInfoExcelDTO entInfo, List<BplcExcelDTO> bplcList) throws Exception {
        // 기본 세팅
        Workbook workbook = new SXSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        int rowIndex = 0;

        // 개별 설정
        ExcelRenderResource mainResource = ExcelRenderResourceFactory.prepareRenderResource(EntInfoExcelDTO.class);
        List<EntInfoExcelDTO> mainDataList = new ArrayList<>();
        mainDataList.add(entInfo);

        ExcelRenderResource bplcResource = ExcelRenderResourceFactory.prepareRenderResource(BplcExcelDTO.class);

        // 엑셀 그리기
        ExcelCreateService<EntInfoExcelDTO> mainExcel = new ExcelCreateService(workbook, sheet, mainDataList, mainDataList.getClass());
        rowIndex = mainExcel.createCaption(rowIndex, "기본정보");
        rowIndex = mainExcel.createVerticalByEntInfo(mainResource, mainDataList, rowIndex);

        rowIndex = mainExcel.createHr(rowIndex); // 구분선
        rowIndex = mainExcel.createCaption(rowIndex, "주요사업장");
        ExcelCreateService<BplcExcelDTO> bplcExcel = new ExcelCreateService(workbook, sheet, bplcList, bplcList.getClass());
        bplcExcel.createVertical(bplcResource, bplcList, rowIndex);

        // 엑셀 사이즈 지정
        sheetSize(sheet, 2);

        // 엑셀 파일 출력
        writeExcel(response, workbook, mainResource);
    }

    // 통계관리
    @Override
    public void cntnStatsExcelDownload(HttpServletResponse response, List<CntnStatsExcelDTO> mainDataList, String chart) throws Exception {
        // 기본 세팅
        Workbook workbook = new SXSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        int rowIndex = 0;

        // 개별 설정
        ExcelRenderResource mainResource = ExcelRenderResourceFactory.prepareRenderResource(CntnStatsExcelDTO.class);

        // 엑셀 그리기
        ExcelCreateService<CntnStatsExcelDTO> mainExcel = new ExcelCreateService(workbook, sheet, mainDataList, mainDataList.getClass());
        mainExcel.chartDraw(sheet, chart, rowIndex, mainResource.getDataFieldNames().size() + 1);
        rowIndex = mainExcel.createHorizontal(mainResource, mainDataList, rowIndex);

        // 엑셀 사이즈 지정
        int cellNumber = mainResource.getDataFieldNames().size();
        sheetSize(sheet, cellNumber);

        // 엑셀 파일 출력
        writeExcel(response, workbook, mainResource);
    }

    // 기업정보 전체 다운로드
    public void entInfoExcelDownloadAll(HttpServletResponse response, List<EntInfoExcelDTO> mainDataList) throws Exception {
        // 기본 세팅
        Workbook workbook = new SXSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        int rowIndex = 0;

        // 개별 설정
        ExcelRenderResource mainResource = ExcelRenderResourceFactory.prepareRenderResource(EntInfoExcelDTO.class);

        // 엑셀 그리기
        ExcelCreateService<EntInfoExcelDTO> mainExcel = new ExcelCreateService(workbook, sheet, mainDataList, mainDataList.getClass());
        mainExcel.createByEntInfoAll(mainResource, mainDataList, rowIndex);

        // 엑셀 사이즈 지정
        int cellNumber = mainResource.getDataFieldNames().size();
        sheetSize(sheet, cellNumber);

        // 엑셀 파일 출력
        writeExcel(response, workbook, mainResource);
    }

    public void sheetSize(Sheet sheet, int cellNumber){
        ((SXSSFSheet)sheet).trackAllColumnsForAutoSizing();
        for(int k = 0 ; k < cellNumber + 1; k++){
            sheet.autoSizeColumn(k);
            sheet.setColumnWidth(k, Math.min(150*150, sheet.getColumnWidth(k) + 1024));
        }
    }
}