package com.takensoft.common.file.service.Impl;

import com.takensoft.common.file.dao.FileMngDAO;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.service.FileService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.26
 *
 * 파일매니저 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * FileMngService : 검토사항 인터페이스 상속
 */
@Service("fileMngService")
@RequiredArgsConstructor
public class FileMngServiceImpl extends EgovAbstractServiceImpl implements FileMngService {
    private final FileMngDAO fileMngDAO;
    private final IdgenService fileMngIdgn;
    private final FileService fileService;
    private final JWTUtil jwtUtil;

    /**
     * @author 박정하
     * @since 2024.03.26
     *
     * 파일매니저 등록
     */
    @Override
    public HashMap<String, Object> fileMngInsert(FileMngVO fileMngVO, List<MultipartFile> multipartFileList) throws Exception {
        HashMap<String, Object> result = new HashMap<>();
        result.put("fileInsertResult", 0);

        // 파일관리 아이디 생성
        String fileMngId = fileMngIdgn.getNextStringId();
        fileMngVO.setFileMngId(fileMngId);

        // 작성자 생성
        String writer = jwtUtil.getWriter();
        fileMngVO.setRgtr(writer);

        // 파일 등록
        try {
            result.put("fileInsertResult", fileService.fileInsert(fileMngId, multipartFileList));
        } catch (IOException e) {
            e.printStackTrace();
            return result;
        }

        // 파일매니저 아이디 삽입
        result.put("fileMngId", fileMngId);

        // DB 등록
        result.put("fileMngResult", fileMngDAO.fileMngInsert(fileMngVO));

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     *
     * 파일매니저 수정
     */
    @Override
    public int fileMngUpadate(FileMngVO fileMngVO, List<HashMap<String, Object>> deleteFileList, List<MultipartFile> multipartFileList) throws Exception {
        int result = 0;

        // 수정자 생성
        String writer = jwtUtil.getWriter();
        fileMngVO.setMdfr(writer);

        // 파일 추가
        if(multipartFileList != null && multipartFileList.size() > 0) {
            // DB 등록
            try {
                String fileMngId =fileMngVO.getFileMngId();
                result += fileService.fileInsert(fileMngId, multipartFileList);
            } catch (IOException e) {
                e.printStackTrace();
                return result;
            }
        }
        
        // 파일 삭제
        if(deleteFileList != null && deleteFileList.size() > 0) {
            try {
                result += fileService.fileDelete(deleteFileList);
            } catch (IOException e) {
                e.printStackTrace();
                return result;
            }
        }
        
        // DB 등록
        result += fileMngDAO.fileMngUpadate(fileMngVO);

        return result;
    }


    /**
     * @author 박정하
     * @since 2024.03.27
     *
     * 파일매니저 삭제
     */
    @Override
    public int fileMngDelete(HashMap<String, Object> params) throws Exception {
        int result = 0;

        // 파일 삭제
        try {
            List<HashMap<String, Object>> fileList = fileService.fileSelectList(params.get("fileMngId").toString());
            result += fileService.fileDelete(fileList);
        } catch (IOException e) {
            e.printStackTrace();
            return result;
        }

        // DB 등록
        result += fileMngDAO.fileMngDelete(params);

        return result;
    }
}