package com.takensoft.portal.entInfo.service.Impl;

import com.takensoft.common.util.JWTUtil;
import com.takensoft.portal.entInfo.dao.BplcDAO;
import com.takensoft.portal.entInfo.service.BplcService;
import com.takensoft.portal.entInfo.vo.BplcVO;
import com.takensoft.portal.entInfo.vo.EntInfoVO;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author  : 박정하
 * @since   : 2024.06.10
 *
 * 기업주요사업장 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * BplcService : 기업주요사업장 인터페이스 상속
 */
@Service("bplcService")
@RequiredArgsConstructor
public class BplcServiceImpl extends EgovAbstractServiceImpl implements BplcService {
    private final JWTUtil jwtUtil;
    private final BplcDAO bplcDAO;

    /**
     * @author 박정하
     * @since 2024.06.10
     *
     * 기업주요사업장 등록
     */
    @Override
    public int bplcListInsert(List<BplcVO> bplcList) throws Exception {
        int result = 0;
        for (BplcVO bplcVO : bplcList) {
            if (bplcVO.getBplcNm() != null) {
                result += bplcDAO.bplcInsert(bplcVO);
            }
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.10
     *
     * 기업주요사업장 수정
     */
    @Override
    public int bplcListUpdate(List<BplcVO> bplcList) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();

        int result = 0;
        for (BplcVO bplcVO : bplcList) {
            if (bplcVO.getBplcNm() != null) {
                // 기업주요사업장 아이디가 있는 경우 (수정)
                if (bplcVO.getBplcId() != null) {
                    bplcVO.setMdfr(writer);
                    result += bplcDAO.bplcUpdate(bplcVO);
                }
                // 기업주요사업장 아이디가 없는 경우 (추가)
                else {
                    bplcVO.setRgtr(writer);
                    result += bplcDAO.bplcInsert(bplcVO);
                }
            }
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.10
     *
     * 기업주요사업장 삭제
     */
    @Override
    public int bplcListDelete(List<BplcVO> bplcDeleteList) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();

        int result = 0;
        for (BplcVO bplcVO : bplcDeleteList) {
            bplcVO.setMdfr(writer);
            result += bplcDAO.bplcUpdate(bplcVO);
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.10
     *
     * 기업주요사업장 삭제 (기업정보 삭제용)
     */
    @Override
    public int bplcDeleteByEntInfoId(String entMttrId) throws Exception {
        EntInfoVO entInfoVO = new EntInfoVO();

        // 수정자 생성
        String writer = jwtUtil.getWriter();
        entInfoVO.setMdfr(writer);

        int result = bplcDAO.bplcUpdateByEntInfoId(entInfoVO);

        return result;
    }
}