package com.takensoft.portal.ivstDscsn.web;

import com.takensoft.common.excel.service.ExcelService;
import com.takensoft.common.util.ResponseData;
import com.takensoft.portal.ivstDscsn.dto.IvstDscsnExcelDTO;
import com.takensoft.portal.ivstDscsn.service.IvstDscsnService;
import com.takensoft.portal.ivstDscsn.vo.IvstDscsnVO;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.nio.charset.Charset;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.26
 *
 * 투자상담 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/government/ivstDscsn")
public class IvstDscsnController {
    private final IvstDscsnService ivstDscsnService;
    private final ExcelService excelService;

    /**
     * @author 박정하
     * @since 2024.03.26
     * @param ivstDscsnVO
     * @return
     * @throws Exception
     *
     * 투자상담 등록
     */
    @PostMapping(path = "/insertProc.file")
    public ResponseEntity<?> insertProc(IvstDscsnVO ivstDscsnVO) throws Exception {
        HashMap<String, Object> result = ivstDscsnService.ivstDscsnInsert(ivstDscsnVO);
        int insertResult = (int)result.get("result");

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if (insertResult > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 투자상담 등록되었습니다.");
            responseData.setData(result);
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else if(insertResult < 0) {
            responseData.setStatus(HttpStatus.PAYLOAD_TOO_LARGE);
            responseData.setMessage("업로드 가능한 파일 용량 제한을 초과했습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.PAYLOAD_TOO_LARGE);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("투자상담 등록에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.26
     * @param params
     * @return
     * @throws Exception
     *
     * 투자상담 목록 조회
     */
    @PostMapping(path = "/listProc.json")
    public ResponseEntity<?> listProc(@RequestBody Map<String, String> params) throws Exception {
        Map<String, Object> result = ivstDscsnService.ivstDscsnList(params);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 투자상담 목록 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.03.26
     * @param ivstDscsnVO
     * @return
     * @throws Exception
     *
     * 투자상담 상세 조회
     */
    @PostMapping(path = "/detailProc.json")
    public ResponseEntity<?> detailProc(@RequestBody IvstDscsnVO ivstDscsnVO) throws Exception {
        IvstDscsnVO result = ivstDscsnService.ivstDscsnDetail(ivstDscsnVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 투자상담 상세 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     * @param ivstDscsnVO
     * @return
     * @throws Exception
     *
     * 투자상담 수정
     */
    @PostMapping(path = "/updateProc.file")
    public ResponseEntity<?> updateProc(IvstDscsnVO ivstDscsnVO) throws Exception {
        int result = ivstDscsnService.ivstDscsnUpdate(ivstDscsnVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if (result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 투자상담 수정 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else if(result < 0) {
            responseData.setStatus(HttpStatus.PAYLOAD_TOO_LARGE);
            responseData.setMessage("업로드 가능한 파일 용량 제한을 초과했습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.PAYLOAD_TOO_LARGE);
        } else {
            responseData.setStatus(HttpStatus.BAD_REQUEST);
            responseData.setMessage("투자상담 수정에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     * @param ivstDscsnVO
     * @return
     * @throws Exception
     *
     * 투자상담 삭제
     */
    @PostMapping(path = "/deleteProc.json")
    public ResponseEntity<?> deleteProc(@RequestBody IvstDscsnVO ivstDscsnVO) throws Exception {
        int result = ivstDscsnService.ivstDscsnDelete(ivstDscsnVO);

        // 응답처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if (result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 투자상담 삭제 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.BAD_REQUEST);
            responseData.setMessage("투자상담 삭제에 실패했습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.27
     *
     * 투자상담 엑셀 다운로드
     */
    @RequestMapping(path = "/excelDownload.json")
    public void excelDownload(HttpServletResponse response, @RequestBody IvstDscsnExcelDTO ivstDscsn) throws Exception {
        excelService.ivstDscsnExcelDownload(response, ivstDscsn);
    }
}