package com.takensoft.portal.rvwMttr.service.Impl;

import com.takensoft.common.util.JWTUtil;
import com.takensoft.portal.rvwMttr.dao.RvwMttrPrgrsDAO;
import com.takensoft.portal.rvwMttr.service.RvwMttrPrgrsService;
import com.takensoft.portal.rvwMttr.vo.RvwMttrPrgrsVO;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.29
 *
 * 검토사항진행 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * RvwMttrPrgrsService : 검토사항진행 인터페이스 상속
 */
@Service("RvwMttrPrgrsService")
@RequiredArgsConstructor
public class RvwMttrPrgrsServiceImpl extends EgovAbstractServiceImpl implements RvwMttrPrgrsService {
    private final RvwMttrPrgrsDAO rvwMttrPrgrsDAO;
    private final JWTUtil jwtUtil;

    /**
     * @author 박정하
     * @since 2024.04.25
     *
     * 검토사항진행 등록
     */
    @Override
    public int prgrsListInsert(String rvwMttrId, List<RvwMttrPrgrsVO> prgrsList) throws Exception {
        int result = 0;
        for (RvwMttrPrgrsVO rvwMttrPrgrsVO : prgrsList) {
            rvwMttrPrgrsVO.setRvwMttrId(rvwMttrId);

            String writer = jwtUtil.getWriter();
            rvwMttrPrgrsVO.setRgtr(writer);

            result +=  rvwMttrPrgrsDAO.prgrsInsert(rvwMttrPrgrsVO);
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.04.25
     *
     * 검토사항진행 수정
     */
    @Override
    public int prgrsListUpdate(List<RvwMttrPrgrsVO> prgrsList) throws Exception {
        int result = 0;

        for (RvwMttrPrgrsVO rvwMttrPrgrsVO : prgrsList) {
            String writer = jwtUtil.getWriter();
            // 검토사항 진행 아이디가 있는 경우 (수정)
            if (rvwMttrPrgrsVO.getRvwMttrPrgrsId() != null) {
                rvwMttrPrgrsVO.setMdfr(writer);
                result += rvwMttrPrgrsDAO.prgrsUpdate(rvwMttrPrgrsVO);
            }
            // 검토사항진행 아이디가 없는 경우 (추가)
            else {
                rvwMttrPrgrsVO.setRgtr(writer);
                result +=  rvwMttrPrgrsDAO.prgrsInsert(rvwMttrPrgrsVO);
            }
        }

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.04.01
     *
     * 검토사항진행 삭제
     */
    public int prgrsListDelete(List<RvwMttrPrgrsVO> deleteList) throws Exception {
        int result = 0;
        for (RvwMttrPrgrsVO rvwMttrPrgrsVO : deleteList) {
            result +=  rvwMttrPrgrsDAO.prgrsDelete(rvwMttrPrgrsVO.getRvwMttrPrgrsId());
        }
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.04
     *
     * 검토사항진행 삭제 (검토사항 삭제용)
     */
    public int prgrsDeleteForRvwMttr(String rvwMttrId) throws Exception {
        return rvwMttrPrgrsDAO.prgrsDeleteForRvwMttr(rvwMttrId);
    }
}