package com.takensoft.portal.rvwMttr.service.Impl;

import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.codeManage.vo.CodeManageVO;
import com.takensoft.common.Pagination;
import com.takensoft.common.file.service.FileMngService;
import com.takensoft.common.file.service.FileService;
import com.takensoft.common.file.vo.FileMngVO;
import com.takensoft.common.idgen.service.IdgenService;
import com.takensoft.common.util.JWTUtil;
import com.takensoft.portal.ivstDscsn.dao.IvstDscsnDAO;
import com.takensoft.portal.ivstDscsn.service.Impl.IvstDscsnServiceImpl;
import com.takensoft.portal.ivstDscsn.vo.IvstDscsnVO;
import com.takensoft.portal.rvwMttr.dao.RvwMttrDAO;
import com.takensoft.portal.rvwMttr.service.RvwMttrPrgrsService;
import com.takensoft.portal.rvwMttr.service.RvwMttrService;
import com.takensoft.portal.rvwMttr.vo.RvwMttrPrgrsVO;
import com.takensoft.portal.rvwMttr.vo.RvwMttrVO;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.29
 *
 * 검토사항 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * RvwMttrService : 검토사항 인터페이스 상속
 */
@Service("rvwMttrService")
@RequiredArgsConstructor
public class RvwMttrServiceImpl extends EgovAbstractServiceImpl implements RvwMttrService {
    private final JWTUtil jwtUtil;
    private final IdgenService rvwMttrIdgn;
    private final CodeManageService codeManageService;
    private final RvwMttrDAO rvwMttrDAO;
    private final RvwMttrPrgrsService rvwMttrPrgrsService;
    private final FileService fileService;
    private final FileMngService fileMngService;
    private final IvstDscsnDAO ivstDscsnDAO;

    /**
     * @author 박정하
     * @since 2024.03.29
     *
     * 검토사항 등록
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public HashMap<String, Object> rvwMttrInsert(RvwMttrVO rvwMttrVO, List<MultipartFile> insertFileList) throws Exception {
        // 아이디 생성
        String rvwMttrId = rvwMttrIdgn.getNextStringId();
        rvwMttrVO.setRvwMttrId(rvwMttrId);

        // 작성자 생성
        String writer = jwtUtil.getWriter();
        rvwMttrVO.setRgtr(writer);

        // 첨부파일 등록
        if (insertFileList != null && !insertFileList.isEmpty() && insertFileList.size() > 0) {
            FileMngVO fileMngVO = new FileMngVO();

            // 파일매니저 등록
            HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);

            // 파일매니저 아이디 삽입
            rvwMttrVO.setFileMngId(fileMng.get("fileMngId").toString());
        }

        // 검토사항 등록
        int rvwMttrResult = rvwMttrDAO.rvwMttrInsert(rvwMttrVO);

        // 검토사항진행 등록
        int prgrsResult = rvwMttrPrgrsService.prgrsListInsert(rvwMttrId, rvwMttrVO.getPrgrsList());

        // 결과 반환
        HashMap<String, Object> result = new HashMap<>();
        result.put("rvwMttrId", rvwMttrId);
        result.put("result", prgrsResult + rvwMttrResult);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.03.29
     *
     * 검토사항 목록 조회
     */
    @Override
    public Map<String, Object> rvwMttrSelectList(Map<String, String> params) throws Exception {
        Pagination search = new Pagination(0, params);
        int cnt = rvwMttrDAO.rvwMttrSelectListCount(search);
        Pagination pagination = new Pagination(cnt, params);
        List<HashMap<String, Object>> list = rvwMttrDAO.rvwMttrSelectList(pagination);

        // 검토사항 유형
        List<CodeManageVO> rvwMttrTypeList = codeManageService.findByChildCdCache("rvwMttrType");

        // 결과 반환
        Map<String, Object> result = new HashMap<>();
        result.put("list", list);
        result.put("pagination", pagination);
        result.put("rvwMttrTypeList", rvwMttrTypeList);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.04
     *
     * 검토사항 상세 조회
     */
    @Override
    public Map<String, Object> rvwMttrDetail(RvwMttrVO rvwMttrVO) throws Exception {
        RvwMttrVO newRvwMttrVO = new RvwMttrVO();

        // RvwMttrId가 있는 경우
        if (rvwMttrVO.getRvwMttrId() != null) {
            newRvwMttrVO = rvwMttrDAO.selectRvwMttrDetail(rvwMttrVO.getRvwMttrId());
            // FileMngId가 있는 경우
            if (newRvwMttrVO.getFileMngId() != null) {
                List<HashMap<String, Object>> fileList = fileService.fileSelectList(newRvwMttrVO.getFileMngId());
                newRvwMttrVO.setFileList(fileList);
            }
        }
        // RvwMttrId가 없는 경우
        else {
            newRvwMttrVO.setType(""); // 유형
            newRvwMttrVO.setUseYn("Y"); // 사용유무
            if (rvwMttrVO.getIvstDscsnId() != null && !rvwMttrVO.getIvstDscsnId().equals("")) {
                String ivstDscsnId = rvwMttrVO.getIvstDscsnId();
                newRvwMttrVO.setIvstDscsnId(ivstDscsnId);
                IvstDscsnVO ivstDscsnVO = ivstDscsnDAO.selectIvstDscsnDetail(ivstDscsnId);
                newRvwMttrVO.setRcptCn(ivstDscsnVO.getMainCn());
            }
            if (rvwMttrVO.getTtl() != null && !rvwMttrVO.getTtl().equals("")) {
                newRvwMttrVO.setTtl(rvwMttrVO.getTtl());
            }
        }

        // PrgrsList가 비어있는 경우
        if (newRvwMttrVO.getPrgrsList().size() == 0) {
            RvwMttrPrgrsVO rvwMttrPrgrsVO = new RvwMttrPrgrsVO();
            rvwMttrPrgrsVO.setRvwMttrId(rvwMttrVO.getRvwMttrId()); // 검토사항 Id
            newRvwMttrVO.getPrgrsList().add(rvwMttrPrgrsVO);
        }

        // 검토사항 유형
        List<CodeManageVO> rvwMttrTypeList = codeManageService.findByChildCdCache("rvwMttrType");

        // 결과 반환
        Map<String, Object> result = new HashMap<>();
        result.put("rvwMttrVO", newRvwMttrVO);
        result.put("rvwMttrTypeList", rvwMttrTypeList);
        return result;
    }

    /**
     * @author 박정하
     * @since 2024.04.01
     *
     * 검토사항 수정
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int rvwMttrUpdate(RvwMttrVO rvwMttrVO, List<MultipartFile> insertFileList) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();
        rvwMttrVO.setMdfr(writer);

        int result = 0;

        // 첨부파일 정보
        String fileMngId = rvwMttrVO.getFileMngId();
        FileMngVO fileMngVO = new FileMngVO();
        List<HashMap<String, Object>> deleteFileList = rvwMttrVO.getDeleteFileList();

        // FileMngId가 있는 경우
        if (fileMngId != null) {
            fileMngVO.setFileMngId(rvwMttrVO.getFileMngId());
            for (HashMap<String, Object> file : deleteFileList) {
                file.put("fileId", Integer.parseInt(file.get("fileId").toString()));
            }
            result += fileMngService.fileMngUpadate(fileMngVO, deleteFileList, insertFileList);
        }
        // FileMngId가 없는 경우
        else {
            if (insertFileList != null && insertFileList.size() > 0) {
                HashMap<String, Object> fileMng = fileMngService.fileMngInsert(fileMngVO, insertFileList);
                result += (int)fileMng.get("fileMngResult");
                rvwMttrVO.setFileMngId(fileMng.get("fileMngId").toString());
            }
        }

        // 검토사항 수정
        result += rvwMttrDAO.rvwMttrUpdate(rvwMttrVO);

        // 검토사항진행 수정
        List<RvwMttrPrgrsVO> prgrsList = rvwMttrVO.getPrgrsList();
        if (prgrsList.size() > 0 && !prgrsList.isEmpty()) {
            result += rvwMttrPrgrsService.prgrsListUpdate(prgrsList);
        }
        // 검토사항진행 삭제
        List<RvwMttrPrgrsVO> deleteList = rvwMttrVO.getDeleteList();
        if (deleteList.size() > 0 && !deleteList.isEmpty()) {
            result += rvwMttrPrgrsService.prgrsListDelete(deleteList);
        }

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.04.01
     *
     * 검토사항 삭제
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int rvwMttrDelete(RvwMttrVO rvwMttrVO) throws Exception {
        // 수정자 생성
        String writer = jwtUtil.getWriter();
        rvwMttrVO.setMdfr(writer);

        // 검토사항 삭제
        int result = 0;
        rvwMttrVO.setUseYn("N");
        result += rvwMttrDAO.rvwMttrDelete(rvwMttrVO);

        // 검토사항진행 삭제
        result += rvwMttrPrgrsService.prgrsDeleteForRvwMttr(rvwMttrVO.getRvwMttrId());

        return result;
    }

    /**
     * @author 박정하
     * @since 2024.06.12
     *
     * 검토사항 삭제 (검토사항 삭제용)
     */
    public int rvwMttrDeleteByIvstDscsnId(String ivstDscsnId) throws Exception {
        int result = 0;

        List<RvwMttrVO> rvwMttrList = rvwMttrDAO.rvwMttrListByIvstDscsnId(ivstDscsnId);
        if (rvwMttrList.size() > 0) {
            for (RvwMttrVO rvwMttrVO : rvwMttrList) {
                result += rvwMttrDelete(rvwMttrVO);
            }
        }

        return result;
    }
}