package com.takensoft.portal.rvwMttr.web;

import com.takensoft.common.excel.service.ExcelService;
import com.takensoft.common.util.ResponseData;
import com.takensoft.portal.rvwMttr.dto.RvwMttrExcelDTO;
import com.takensoft.portal.rvwMttr.dto.RvwMttrPrgrsExcelDTO;
import com.takensoft.portal.rvwMttr.service.RvwMttrService;
import com.takensoft.portal.rvwMttr.vo.RvwMttrVO;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.nio.charset.Charset;
import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.03.29
 *
 * 검토사항 관련 컨트롤러
 */
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/government/rvwMttr")
public class RvwMttrController {
    private final RvwMttrService rvwMttrService;
    private final ExcelService excelService;

    /**
     * @author 박정하
     * @since 2024.03.29
     * @param rvwMttrVO
     * @return
     * @throws Exception
     *
     * 검토사항 등록
     */
    @PostMapping(path = "/insertProc.file")
    public ResponseEntity<?> insertProc(@RequestPart RvwMttrVO rvwMttrVO, @RequestPart(required = false) List<MultipartFile> insertFileList) throws Exception {
        HashMap<String, Object> result = rvwMttrService.rvwMttrInsert(rvwMttrVO, insertFileList);
        int insertResult = (int)result.get("result");

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if(insertResult > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 검토사항 등록 처리되었습니다.");
            responseData.setData(result);
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else if(insertResult < 0) {
            responseData.setStatus(HttpStatus.PAYLOAD_TOO_LARGE);
            responseData.setMessage("업로드 가능한 파일 용량 제한을 초과했습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.PAYLOAD_TOO_LARGE);
        } else {
            responseData.setStatus(HttpStatus.INTERNAL_SERVER_ERROR);
            responseData.setMessage("검토사항 등록에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * @author 박정하
     * @since 2024.03.29
     * @param params
     * @return
     * @throws Exception
     *
     * 검토사항 목록 조회
     */
    @PostMapping(path = "/listProc.json")
    public ResponseEntity<?> listProc(@RequestBody Map<String, String> params) throws Exception {
        Map<String, Object> result = rvwMttrService.rvwMttrSelectList(params);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 검토사항 목록 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.06.04
     * @param rvwMttrVO
     * @return
     * @throws Exception
     *
     * 검토사항 상세 조회
     */
    @PostMapping(path = "/detailProc.json")
    public ResponseEntity<?> detailProc(@RequestBody RvwMttrVO rvwMttrVO) throws Exception {
        Map<String, Object> result = rvwMttrService.rvwMttrDetail(rvwMttrVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        responseData.setStatus(HttpStatus.OK);
        responseData.setMessage("정상적으로 검토사항 상세 조회가 처리되었습니다.");
        responseData.setData(result);
        return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
    }

    /**
     * @author 박정하
     * @since 2024.04.01
     * @param rvwMttrVO
     * @return
     * @throws Exception
     *
     * 검토사항 수정
     */
    @PostMapping(path = "/updateProc.file")
    public ResponseEntity<?> updateProc(@RequestPart RvwMttrVO rvwMttrVO, @RequestPart(required = false) List<MultipartFile> insertFileList) throws Exception {
        int result = rvwMttrService.rvwMttrUpdate(rvwMttrVO, insertFileList);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if (result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 검토사항 수정 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.BAD_REQUEST);
            responseData.setMessage("검토사항 수정에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * @author 박정하
     * @since 2024.04.01
     * @param rvwMttrVO
     * @return
     * @throws Exception
     *
     * 검토사항 삭제
     */
    @PostMapping(path = "/deleteProc.json")
    public ResponseEntity<?> deleteProc(@RequestBody RvwMttrVO rvwMttrVO) throws Exception {
        // 검토사항 삭제
        int result = rvwMttrService.rvwMttrDelete(rvwMttrVO);

        // 응답 처리
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(new MediaType("application", "json", Charset.forName("UTF-8")));
        ResponseData responseData = new ResponseData();
        if (result > 0) {
            responseData.setStatus(HttpStatus.OK);
            responseData.setMessage("정상적으로 검토사항 삭제 처리되었습니다.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.OK);
        } else {
            responseData.setStatus(HttpStatus.BAD_REQUEST);
            responseData.setMessage("검토사항 삭제에 실패하였습니다.\n담당자에게 문의하세요.");
            return new ResponseEntity<>(responseData, headers, HttpStatus.BAD_REQUEST);
        }
    }

    /**
     * @author 박정하
     * @since 2024.04.01
     *
     * 검토사항 엑셀 다운로드
     */
    @RequestMapping(path = "/excelDownload.file")
    public void excelDownload(HttpServletResponse response, @RequestPart RvwMttrExcelDTO rvwMttr, @RequestPart List<RvwMttrPrgrsExcelDTO> prgrsList) throws Exception {
        excelService.rvwMttrExcelDownload(response, rvwMttr, prgrsList);
    }
}