package com.takensoft.portal.statistics.service.Impl;

import com.takensoft.cms.codeManage.service.CodeManageService;
import com.takensoft.cms.codeManage.vo.CodeManageVO;
import com.takensoft.portal.statistics.dao.StatisticsDAO;
import com.takensoft.portal.statistics.service.StatisticsService;
import lombok.RequiredArgsConstructor;
import org.egovframe.rte.fdl.cmmn.EgovAbstractServiceImpl;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author  : 박정하
 * @since   : 2024.06.13
 *
 * 통계 관련 구현체
 * EgovAbstractServiceImpl : 전자정부 상속
 * PopupService : 통계 관련 인터페이스 상속
 */
@Service("statisticsService")
@RequiredArgsConstructor
public class StatisticsServiceImpl extends EgovAbstractServiceImpl implements StatisticsService {
    private final StatisticsDAO statisticsDAO;
    private final CodeManageService codeManageService;

    /**
     * @author  : 박정하
     * @since   : 2024.06.13
     *
     * 분류별 통계 조회
     */
    public List<HashMap<String, Object>> categoryStatistics(HashMap<String, String> params) throws Exception {
        return statisticsDAO.categoryStatistics(params);
    }

    /**
     * @author  : 박정하
     * @since   : 2024.06.13
     *
     * 검토사항 통계 조회
     */
    public Map<String, Object> rvwMttrStatistics(HashMap<String, String> params) throws Exception {
        List<HashMap<String, Object>> statisticsList = statisticsDAO.rvwMttrStatistics(params);

        // 검토사항 유형
        List<CodeManageVO> rvwMttrTypeList = codeManageService.findByChildCdCache("rvwMttrType");

        // 결과 반환
        Map<String, Object> result = new HashMap<>();
        result.put("statisticsList", statisticsList);
        result.put("rvwMttrTypeList", rvwMttrTypeList);
        return result;
    }

    /**
     * @author  : 박정하
     * @since   : 2024.06.14
     *
     * 상담신청 통계 조회
     */
    public List<HashMap<String, Object>> consultationStatistics(HashMap<String, String> params) throws Exception {
        return statisticsDAO.consultationStatistics(params);
    }
}